(function(App){

    App.modules = {};
    App.services = {};
    App.views = {};
    App.mixins = {};

    /* start modules and switch between pages */
    App.init = function(conf) {
        var self = this;

        console.log(conf.client.appName + ' v' + conf.ver);
        console.log(conf.client.website);

        _.bindAll(self, 'pageShow', 'pageBeforeHide', 'pageReload', 'pageCreate', 'setSettingClass', 'systemLinkClick',
            'initNotificationsOnFirstLaunch', 'favSportsChange', 'changeContainer', 'openNavigation', 'initMainPage',
            'externalLinkClick');

        window.StatusBar && StatusBar.overlaysWebView(false) && StatusBar.show();

        self.pageContainer = $.mobile.pageContainer;
        self.conf = conf;
        self.currentSport = 0;

        self.defaultModule = 'schedule';
        self.currentModule = undefined;
        self.currentModule = self.defaultModule;

        self.appInterval = undefined;
        self.pages = []; // main pages
        self.isGAEnabled = false;
        self.isOffline = false;
        self.pageInitialized = false;
        self.defaultThumbnail = self.conf.client.website + self.conf.defaultThumbnail;
        self.requestInProgress = undefined;
        self.platform = undefined;
        self.firstLaunch = true;
        self.ptr = $('.ptr');

        self.setPlatform();

        self.defaultTemplateData = {
            addBackButton : false,
            refreshable : false,
            headerSportFav : false,
            platform : self.platform,
            client : self.conf.client,
            minorVer : self.conf.minorVer,
            ver : self.conf.ver,
            packageIdPrefix : self.conf.packageIdPrefix
        };

        // detect if app is running for the first time
        if (window.localStorage.getItem('wasLaunched')) {
            self.firstLaunch = false;
        } else {
            window.localStorage.setItem('wasLaunched', true);
        }

        // disable page transitions for Android
        self.slideTransition = 'slide';
        if (self.platform == 'android') {
            $.mobile.defaultPageTransition = 'none';
            self.slideTransition = 'none';
        }

        self.renderStaticPages();

        // initializing services
        self.services.notifications.init();
        self.services.loader.init();
        self.services.favSports.init();
        self.services.moreNavbar.init();
        self.services.settings.init();

        //initializing modules
        _.each(self.modules, function(e, i) {
            e.init(i == self.defaultModule);
        });

        // Notifications workflow on app first launch
        if (self.firstLaunch && ! App.services.notifications.hasSettings) {
            $.subscribe('/notifications/deviceid', self.initNotificationsOnFirstLaunch);
        }

        // initializing Google Analytics plugin
        if (App.conf.client.GA) {
            if (window.plugins && window.plugins.gaPlugin) {
                window.plugins.gaPlugin.init(function() {
                    self.isGAEnabled = true;
                }, function(e){}, App.conf.client.GA, 10);
            }
        }

        //schedule and headlines content reloader
        self.appInterval = setInterval(function() {
            self.reloadAllPages();
        }, self.conf.appUpdatePeriod);

        //online and offline events
        document.addEventListener('online', function() {
            self.checkConnection('online');
        }, true);
        document.addEventListener('offline', function() {
            self.checkConnection('offline');
        }, true);

        // sport changed event
        $.subscribe('/sportmenu/sportselected', function(sport, module) {
            self.switchSport(sport, module);
        });

        //subpages switcher
        $.subscribe('/subpage/ready', function(pageId, pageType, reverse, transition) {
            var transition = transition;
            if ($('#' + pageId).data('transition') === 'slide' && !transition) {
                transition = self.slideTransition;
            }
            self.changePage(pageId, {
                reverse : reverse,
                transition : transition
            });
        });

        // footer buttons handler
        $(document).on('tap', '.main-trigger', function() {
            var module = $(this).data('module');
            if (module && self.modules[module] !== undefined) {
                self.currentModule = module;
                self.modules[module].init(true);
            }
            return false;
        });

        $(document).on('pagecreate', '[data-role=page]', self.pageCreate);
        $(document).on('pageshow', '[data-role=page]', self.pageShow);
        $(document).on('pagereload', '[data-role=page]', self.pageReload);
        $(document).on('pagebeforehide', '[data-role=page]', self.pageBeforeHide);

        // inappbrowser and system browser links
        $(document).on('tap', '.external-link', self.externalLinkClick);
        $(document).on('tap', '.system-browser-link', self.systemLinkClick);

        // switches containers within same page
        $(document).on('tap', '.type-trigger', self.changeContainer);

        // abort any page loads if notification has been received
        $.subscribe('/notifications/received', function() {
            if (self.requestInProgress !== undefined) {
                self.requestInProgress.abort();
            }
        });

        // sports nav triggers
        $(document).on('swiperight', '.ui-page[data-has-panel="true"] .ui-content', self.openNavigation);
        $(document).on('tap', '.icon-list', self.openNavigation);

        $(document).on('tap', '.header .icon-sport-fav', function(e) {
            self.services.favSports.favSport(self.currentSport);
            e.preventDefault();
        });

        // update fav sport icon in header
        $.subscribe('sports/favourite/add', self.favSportsChange);
        $.subscribe('sports/favourite/remove', self.favSportsChange);

        // starting functionality of the page link
        self.pageViews = {};
        $(document).on('tap', '.link', function() {
            var link = $(this),
                params;

            if (! link.data('pageId')) {
                link.data('pageId', self.generatePageId(link.data('view').toLowerCase()));
            }

            if (! self.pageViews[link.data('pageId')]) {

                // grab only data-param values
                params = helpers.dataWildcard(link, 'param');

                self.pageViews[link.data('pageId')] = new self.views[link.data('view') + 'PageView'](_.extend(params, {
                    id : link.data('pageId')
                }));

                self.pageViews[link.data('pageId')].promise.done(function() {
                    self.pageViews[link.data('pageId')].open();
                });

            } else {
                self.pageViews[link.data('pageId')].open();
            }
        });

        $.subscribe('settings/change', function(name, value) {
            App.setSettingClass(value, name);
        });
        _.each(App.services.settings.getSettings(), App.setSettingClass);

        //logger.init();
    }; // App.init

    App.setSettingClass = function(value, name) {
        this.pageContainer.removeClass(function(i, className) {
            var classes = className.split(' '),
                classToRemove = '';

            _.each(classes, function(v, i) {
                if (v.indexOf('setting-' + name) === 0) {
                    classToRemove = v;
                }
            });
            return classToRemove;
        });
        this.pageContainer.addClass('setting-' + name + '-' + value);
    };

    App.renderStaticPages = function() {
        var self = this;

        $('.static-page').each(function(i, e) {
            var pageContainer = $(this),
                backButton = pageContainer.data('backButton') || false,
                backButtonLabel = pageContainer.data('backButtonLabel') || false,
                title = pageContainer.data('title') || self.getCurrentSportTitle(),
                content;

            if (pageContainer.data('template')) {
                content = self.templates[pageContainer.data('template')](self.defaultTemplateData);
            } else {
                content = pageContainer.html();
            }

            page = $(self.templates['main-page']($.extend({}, self.defaultTemplateData, {
                page : content,
                pageId : pageContainer.data('id'),
                title : title,
                footerActiveButton : 'more',
                addBackButton : backButton,
                backButtonLabel : backButtonLabel,
                gaTitle : title,
                pageClass : 'static-page'
            })));

            self.createPage(page);
            pageContainer.remove();
        });
    };

    App.changeType = function(page, type) {
        var container = page.find('.ui-content');

        if (container.find('.type-container').length > 1) {
            container.find('.type-container')
                .addClass('hidden')
                .filter('[data-type=' + type + ']').removeClass('hidden');
        }

        page.data('type', type);
    };

    //change current sport and reinitialize current module
    App.switchSport = function(sport, module, force) {
        if (sport === this.currentSport && module === this.currentModule && ! force) {
            return;
        }

        this.currentSport = sport;
        this.currentModule = module;

        if (this.modules[this.currentModule] !== undefined) {
            this.modules[this.currentModule].init(true);
        } else {
            this.modules[this.defaultModule].init(true);
        }
    };

    App.getSportBySportCode = function(sportCode) {
        var sport;
        _.each(this.conf.client.sports, function(e, i) {
            if (e.code && e.code === sportCode) {
                sport = i;
            }
        });
        return sport;
    };

    // switches sport by sport code, needed for notifications
    App.swtichSportBySportCode = function(sportCode, module) {
        var sportToOpen,
            self = this;

        sportToOpen = self.getSportBySportCode(sportCode);

        if (typeof sportToOpen !== "undefined") {
            if (module) {
                self.currentModule = module;
            }
            self.switchSport(sportToOpen, self.currentModule, true);
        }
    };

    //set images paths to the remote website
    App.adjustImagePaths = function(images) {
        var self = this;

        images.each(function(){
            var src = $(this).attr('src');
            if (! helpers.isExternalURL(src)) {
                $(this).attr('src', self.conf.client.website + src);
            }
        })
    };

    //reloads all pages when device is back online and per interval
    App.reloadAllPages = function(params) {
        var self = this;

        if (self.isOffline) {
            return;
        }

        _.each(self.pages, function(e, i) {
            return self.reloadPage(e);
        });
    };

    App.reloadPage = function(pageId, parser, hideSpinner) {
        var page = $('#' + pageId),
            url = page.data('pageUrl'),
            container = page.find('.page-body');

        if (! url.length) {
            return;
        }

        !hideSpinner && $.publish('loading/started');

        page.trigger('pagebeforereload', [page]);

        return $.get(url, { refresh : Date.now() }, function(data) {
            !hideSpinner && $.publish('loading/stopped');

            if (parser) {
                data = parser(data);
            }
            container.html(data);
            page.trigger('create');
            page.trigger('pagereload', [page]);
        });
    };

    // handle an auto update page, like boxscore or scores tab with in progress game
    App.addAutoUpdatedPage = function(pageId, params, reloadTest) {
        var timer,
            self = this,
            page = $('#' + pageId),
            query,
            data = $.extend({
                parser : null,
                pageId : pageId
            }, params);

        function startTimer() {
            timer = setInterval(function() {
                if (reloadTest === undefined || reloadTest()) {
                    query = self.reloadPage(data.pageId, data.parser);
                }
            }, self.conf.boxscoresUpdatePeriod);
        }

        $(document).on('pageshow', '#' + pageId, function() {
            startTimer();
        });

        if (page == $.mobile.activePage) {
            startTimer();
        }

        $(document).on('pagehide', '#' + pageId, function() {
            clearInterval(timer);
            if (query !== undefined) {
                query.abort();
            }
        });
    };

    App.checkConnection = function(eventType) {
        var connectionOff = (navigator.connection.type == Connection.NONE || navigator.connection.type == Connection.UNKNOWN);

        //lastEventTime && (console.log)(eventType + ' time passed ' + (new Date().getTime() - lastEventTime));
        //!lastEventTime && (console.log(eventType + ' first event fired at' + new Date().getTime()));
        //lastEventTime = new Date().getTime();

        if (eventType === 'offline' && connectionOff) {
            this.showMessage(this.conf.client.appName + this.conf.messages.noConnection);
            this.isOffline = true;
        }

        if (eventType === 'online' && this.isOffline) {
            this.isOffline = false;
            if (this.pages.length > 0) {
                this.reloadAllPages();
                // page back if message is opened
                if ($.mobile.activePage.attr('id') === 'message-page') {
                    $.mobile.activePage.find('.header a[data-rel=back]').click();
                }
            } else {
                //init default page
                this.modules[this.defaultModule].init(true);
            }
        }
    };

    // unified main page loading function
    App.showMainPage = function(params) {
        var self = this;

        if (this.pageExist(params.pageId)) {
            this.changePage(params.pageId);
        } else {
            this.loadMainPage(params, this.getMainPageURL(params.module), this.templates['main-page'])
                .done(function(page) {
                    self.initMainPage(page);
                    // give some time for jqm to finish and show page
                    _.defer(function() {
                        self.changePage(page.attr('id'));
                    });
                });
        }

        self.currentModule = params.module;
    };

    App.loadMainPage = function(params, url, template) {
        var self = this,
            d = $.Deferred();

        if (self.requestInProgress !== undefined) {
            return;
        }

        $.publish('loading/started');

        self.requestInProgress = $.ajax({
            url : url,
            success : function(data) {
                var rendered = $(template($.extend({}, self.defaultTemplateData, {
                    page : data,
                    pageId : params.pageId,
                    title : self.getCurrentSportTitle(),
                    footerActiveButton : params.module,
                    url : url,
                    showStandings : $(data).filter('.type-container[data-type=standings]').length,
                    pageClass : params.module + '-page'
                }, params.tmplData)));

                self.requestInProgress = undefined;

                params.callback && params.callback(rendered);

                $.publish('loading/stopped');
                d.resolve(rendered);
            }
        }).error(function() {
            self.requestInProgress = undefined;
            $.publish('loading/error');
            d.reject();
        });

        return d;
    };

    App.initMainPage = function(page) {
        var pageId = page.attr('id');

        // store page to reload all pages
        if ($.inArray(pageId, this.pages) === -1) {
            this.pages.push(pageId);
        }

        // create navigation
        page.data('navigationView', new this.views.NavigationView({
            el : page.find('[data-role="panel"]')
        }));

        // jqm create page wrapper
        this.createPage(page);
    };

    App.initSubPage = function(params) {
        if (this.pageExist(params.pageId)) {
            if (params.pageExistCallback) {
                params.pageExistCallback($.mobile.activePage, $('#' + params.pageId));
            }
            $.publish('/subpage/ready', [params.pageId, params.pageType, params.reverse, params.transition]);
        } else {
            $.publish('loading/started');
            this.loadSubPage(params);
        }
    };

    App.loadSubPage = function(params) {
        var self = this;

        if (! params.url.length || self.requestInProgress !== undefined) {
            return;
        }

        self.requestInProgress = $.get(params.url, function(data) {
            var rendered;

            $.publish('loading/stopped');

            if (params.parser) {
                data = params.parser(data);
            }

            rendered = $(self.templates['sub-page']($.extend({}, self.defaultTemplateData, {
                pageId : params.pageId,
                page : data,
                url : params.url,
                title : self.getCurrentSportTitle(),
                pageClass : params.pageType + '-page',
                footerActiveButton : self.currentModule
            }, params.tmplData)));

            self.requestInProgress = undefined;

            self.createPage(rendered);

            if (params.callback) {
                params.callback(rendered);
            }

            setTimeout(function() {
                $.publish('/subpage/ready', [params.pageId, params.pageType, params.reverse, params.transition]);
            }, 0);

        }).error(function() {
            self.requestInProgress = undefined;
            $.publish('loading/error');
        });
    };

    // show a message on separate page
    App.showMessage = function(message) {
        $('#message-wrapper h4').html(message);
        this.changePage('message-page');
    };

    App.changePage = function(page, params) {
        var transition = (params && params.transition) ? params.transition : $.mobile.defaultPageTransition,
            $page = $('#' + page);

        // hiding splash screen
        if (! this.pageInitialized) {
            setTimeout(function(){
                if (navigator && navigator.splashscreen) {
                    navigator.splashscreen.hide();
                }
            }, 300);
            this.pageInitialized = true;
        }

        console.log('Page changed to : ' + '#' + page);

        $.mobile.changePage('#' + page, $.extend({
            allowSamePageTransition : true,
            changeHash : true,
            transition : transition
        }, params));

        // track page change in ga, but skip popups
        if (! $page.hasClass('ui-dialog')) {
            this.trackPageChange();
        }
    };

    App.trackPageChange = function() {
        var GAUrl,
            url = $.mobile.activePage.data('pageUrl');

        if (this.isGAEnabled) {
            if (url.length > 0) {
                url = url.substring(0, url.indexOf('?'));
            }
            if ($.mobile.activePage.hasClass('static-page')) {
                GAUrl = $.mobile.activePage.data('gaTitle');
            } else {
                GAUrl = App.getCurrentSportTitle() + ' / ' + $.mobile.activePage.data('gaTitle') + ' / ' + url;
            }
            window.plugins.gaPlugin.trackPage(function(){}, function(){}, GAUrl);
        }
    };

    App.createPage = function(page){
        page.prependTo($.mobile.pageContainer);
        return page.page();
    };

    App.pageExist = function(pageId) {
        return $('#' + pageId).length > 0;
    };

    App.getCurrentModule = function() {
        return this.currentModule;
    };

    App.getCurrentSport = function() {
        return this.currentSport;
    };

    App.getCurrentSportTitle = function() {
        return this.conf.client.sports[this.currentSport].title;
    };

    // creates absolute URL for AJAX usage
    App.normalizeURL = function(url) {
        var website;

        if (! url) {
            return '';
        }

        // replace spaces with %20
        url = url.replace(/\s/g, '%20');

        // handle protocol relative urls
        if (url.indexOf('//') === 0) {
            url = 'http:' + url;
        }

        if (helpers.isExternalURL(url)) {
            return url;
        } else {
            website = this.conf.client.website;
            return website + url;
        }
    };

    // generates urls with client website, current version and a sport module url from client.js
    App.getMainPageURL = function(module) {
        var url = this.conf.client.sports[this.currentSport].urls[module];
        return this.mergeParamsIntoURL(this.conf.client.website + url + (url.indexOf('?') !== -1 ? '&' : '?') + 'tmpl=fr-v' +
            this.conf.ver + '-' + module + '-template', module);
    };

    // URL parameters merging from client.js : global module params + sport specific params
    App.mergeParamsIntoURL = function(url, module) {
        var urlParams = {},
            urlToMerge;

        if (this.conf.client.urlParams && this.conf.client.urlParams[module]) {
            $.extend(urlParams, this.conf.client.urlParams[module]);
        }

        if (this.conf.client.sports[this.currentSport].params) {
            $.extend(urlParams, this.conf.client.sports[this.currentSport].params);
        }

        urlToMerge = $.param(urlParams);

        if (urlToMerge.length > 0) {
            if (url.indexOf('?') !== -1) {
                url += '&' + urlToMerge;
            } else {
                url += '?' + urlToMerge;
            }
        }

        return url;
    };

    App.setPlatform = function() {
        var platformsTests = {
                ios: /(iPhone|iPod|iPad)/,
                android: /Android/
            },
            self = this;

        _.each(platformsTests, function(e, i) {
            if (device.platform.match(e)) {
                self.platform = i;
                return false
            }
        });

        this.pageContainer.addClass(self.platform);
    };

    App.openExternalLink = function(url, opts) {
        var params = $.extend({
            location : 'yes'
        }, opts);
        return window.open(url, '_blank', $.param(params));
    };

    App.openSystemLink = function(url) {
        return window.open(url, "_system");
    };

    App.generatePageId = function(prefix) {
        return _.uniqueId(prefix + '-');
    };

    App.openNavigation = function() {
        $.mobile.activePage.data('navigationView').open();
    };

    App.refreshLayout = function(el) {
        el.trigger('create');
    };

    App.favSportsChange = function(sport) {
        $('#scoreboard-page-' + sport + ', #headlines-page-' + sport).find('.header .icon-sport-fav').toggleClass('icon-sport-fav-active');
    };

    /* page life cycle callbacks */
    App.pageCreate = function(e) {
        var self = this,
            page = $(e.currentTarget),
            wptr;

        // init pull to refresh with reload function
        if (page.find('.ptr-content').length > 0) {
            wptr = new WebPullToRefresh({
                ptrEl: this.ptr.get(0),
                contentEl: page.find('.ptr-content').get(0),
                loadingFunction: function() {
                    return self.reloadPage(page.attr('id'), null, true)
                }
            });
            page.on('panelbeforeopen', function() {
                wptr.isTouchInProgress() && wptr.stop();
            });
        }
    };

    App.pageShow = function(e) {
        var page = $(e.currentTarget),
            activeType = page.data('type') || App.conf.defaultType[this.currentModule];
            //title = page.find('.ui-header .ui-title');

        this.changeType(page, activeType);

        // fix the 1px page content scroll so that pull to refresh works with first attempt
        _.defer(function() {
            $(window).scrollTop(0);
        });

        // show pull to refresh back
        this.ptr.show();

        // check if page has sub-tabs and in this case assign a body class, so pull to refresh positioned correctly
        if (page.find('.header-subtabs').length > 0) {
            this.pageContainer.addClass('has-subtabs');
        } else {
            this.pageContainer.removeClass('has-subtabs');
        }

        //page.removeClass('header-truncated');
        //helpers.isTextTruncated(title.get(0)) && page.addClass('header-truncated');
    };

    App.pageBeforeHide = function(e) {
        var page = $(e.currentTarget);

        // return highlight to current sport in sport nav
        page.data('navigationView') && page.data('navigationView').resetHighlight();

        // hide pull to refresh so it's not visible during transitions
        this.ptr.hide();
    };

    App.pageReload = function(e, page) {
        var activeType = page.data('type') || App.conf.defaultType[self.currentModule];
        this.changeType(page, activeType);
    };

    /* links */
    App.externalLinkClick = function(e) {
        var link = $(e.currentTarget),
            url = link.attr('href'),
            ref,
            location = 'yes',
            self = this;

        if (link.hasClass('link-Audio') || link.hasClass('link-Video')) {
            location = 'no';
        }

        ref = self.openExternalLink(url, {
            location : location
        });

        // let phone go to sleep again once in app browser closed
        ref.addEventListener('exit', function() {
            window.plugins.insomnia.allowSleepAgain();
        });

        // keep phone awake
        window.plugins.insomnia.keepAwake();

        _.defer(function() {
            link.removeClass(self.conf.classes.activeBtn + ' ui-focus')
                .parents('.ui-btn').removeClass(self.conf.classes.activeBtn + ' ui-focus')
        });

        e.preventDefault();
    };

    App.systemLinkClick = function(e) {
        var link = $(e.currentTarget),
            targetURL = link.attr("href");

        this.openSystemLink(targetURL);
        link.removeClass(self.conf.classes.activeBtn + ' ui-focus');

        e.preventDefault();
    };

    App.initNotificationsOnFirstLaunch = function() {
        var self = this;

        (self.requestInProgress !== undefined) && self.requestInProgress.abort();

        self.services.notifications.initNotificationsPage().done(function() {
            // overwriting back button so it points to All Sports Scoreboard
            var notificationsPage = $('#notifications-page'),
                backButton = notificationsPage.find('.header .ui-btn[data-rel="back"]'),
                backButtonTitleEl = backButton.find('.ui-btn-text'),
                backButtonOrigTitle = backButtonTitleEl.text();

            backButton.one('tap', function() {
                self.modules[self.defaultModule].init(true);
                return false;
            });
            backButtonTitleEl.text('Back');

            // restore the back button functionality after notification page is hidden
            notificationsPage.one('pagehide', function(){
                backButtonTitleEl.text(backButtonOrigTitle);
            });

            self.changePage('notifications-page');

            _.delay(function() {
                navigator.notification.alert(self.conf.messages.notificationsFirstLaunch, function(){}, 'Notifications notice');
            }, 1000);
        });
    };

    App.changeContainer = function(e) {
        var page = $.mobile.activePage,
            type = $(e.currentTarget).data('type'),
            currentType = page.data('type') ? page.data('type') : this.conf.defaultScheduleType;

        if (currentType === type) {
            return;
        }

        this.changeType(page, type);
        page.find('.header-subtabs').find('.type-trigger')
            .removeClass(this.conf.classes.persist)
            .filter('[data-type="' + type + '"]').addClass(this.conf.classes.persist);

        e.preventDefault();
    };

    App.removePage = function(pageId) {
        this.pageViews[pageId] && delete this.pageViews[pageId];
    };

})(window.App = window.App || Object.create({}), jQuery);