(function(App, $){

	var schedule = {};
	$.extend(App.modules, { schedule : schedule });
	
	schedule.init = function(load) {
		var self = this,
			schedulePageId = 'scoreboard-page-' + App.currentSport;
			
		if (load) {
			App.showMainPage({
				pageId : schedulePageId, 
				module : 'schedule',
				tmplData : {
					subTabs : (App.conf.client.sports[App.currentSport].params && App.conf.client.sports[App.currentSport].params.allSports) ? false : 'schedule',
                    gaTitle : 'Scoreboard',
                    refreshable : true,
                    headerSportFav : App.currentSport !== 0,
                    isFavourite : _.indexOf(App.services.favSports.getFavSports(), App.currentSport) !== -1
				},
				callback : function(page) {
					self.initializeSchedule(page);
				}
			});
		}
		
		if (! self.initialized) {
			$(document).on('tap', '.boxscore-load', function() {
				var link = $(this);

                if (! link.data('targetPageId')) {
                    link.data('targetPageId', App.generatePageId('boxscore'));
                }

                schedule.boxscore.load(link.data('targetPageId'), link.data('url'), link.data('inProgress') || false);
				
				return false;
			});
									
			// loads schedules, for standings teams
			$(document).on('tap', '.schedule-load', function() {
				var url = App.mergeParamsIntoURL(App.normalizeURL($(this).data('url')), 'schedule'),
					pageId = 'scoreboard-page-' + App.currentSport + '-' + self.cleanTeamName($(this).data('team'));
					
				App.initSubPage({
					pageId : pageId,
					pageType : 'schedule',
					url : url,
					tmplData : {
						backButtonLabel : 'Standings',
						backButtonURL : '#scoreboard-page-' + App.currentSport,
                        gaTitle : $(this).data('team') + '\'s Scoreboard'
					}
				});
				
				return false
			});

            // event details - clicked on other page then event
            $(document).on('tap', '.ui-content .event-load', function() {
                var link = $(this);

                schedule.event.load(link.data('eventid'), link.data('url'), link.data('team'));

                return false;
            });

            // event details - clicked in event tabs
            $(document).on('tap', '.single-event-dates .event-load', function() {
                var link = $(this);

                schedule.event.load(link.data('eventid'), link.data('url'), link.data('team'),
                    (link.data('when') === 'past'),  true);

                return false;
            });

			$(document).on('change', '.schedule-filter', function() {
				var val = $(this).val() || '',
					page = $.mobile.activePage,
					events = page.find('.event-expanded, .event-compact');

				if (val.length) {
					events.addClass('event-hidden')
						.filter('[data-sport="' + val + '"]')
						.removeClass('event-hidden');
				} else {
					events.removeClass('event-hidden');
				}
			});
			 
			self.initialized = true;
		}
	};
	
	schedule.initializeSchedule = function(page) {

        //check if scoreboard has in progress events
		var list = page.find('.schedule-list');
	
		if (list.children('li[data-in-progress=true]').length > 0) {
			App.addAutoUpdatedPage(page.attr('id'), {}, function() {
				return !page.find('.type-container[data-type=scoreboard]').hasClass('hidden');
			});
		}
	};
	
	schedule.cleanTeamName = function(str) {
		return str.replace(/[^a-zA-Z0-9]+/g, '');
	};

	schedule.event = {
        load : function(eventId, rawURL, team, applyReverseTransition, openedFromTabs) {
            var self = this,
                pageId = 'event-page-' + eventId,
                url = App.normalizeURL(rawURL);

            App.initSubPage({
                pageId : pageId,
                pageType : 'event',
                url : url,
                parser : $.proxy(self.parse, self),
                callback : $.proxy(self.pageCreatedCallback, self),
                tmplData : {
                    backButtonLabel : 'Scores',
                    //back button should point to the schedule page, not the previous event
                    backButtonURL : '#scoreboard-page-' + App.currentSport + (team ? '-' + schedule.cleanTeamName(team) : ''),
                    refreshable : true
                },
                // same event can be opened from All Sports scoreboard or from single sport scoreboard,
                // in case event page already in DOM then back button need to be adjusted
                pageExistCallback : function(oldPage, newPage) {
                    if (! openedFromTabs) {
                        newPage.find('.ui-header .ui-btn-left').attr('href', '#' + oldPage.attr('id'));
                    }
                },

				// forse default transition if event page is opened from another kind of page
				transition : openedFromTabs ? App.slideTransition : $.mobile.defaultPageTransition,

                // reverse direction is applicable only when event is in the past relative to the current event,
                // thus reversed direction can not be directly attached to a page,
                // and need to be passed directly to changePage function
                reverse : applyReverseTransition
            });
        },

		// remove empty data containers
		parse : function(html){
			var data = $('<div />').append(html);
			data.find('.single-event-wrapper').each(function() {
				if (! $(this).text().trim()) { 
					$(this).remove();
				}
			});
			
			return data.html();
		},

        pageCreatedCallback : function(page) {
            var links = this.renderTabs(page),
                teamVsTeamText = page.find('.ui-bar p:first-child').map(function() {
                    return $(this).text().trim();
                }).toArray().join(' vs. ');

            // setting GA title in a callback
            page.data('gaTitle', links[1].title.trim() + ' / ' + teamVsTeamText);
        },

		// date tabs on single event page should be implemented as header tabs		 
		renderTabs : function(page) {
			var eventLinks = new Array(3),
				tabs,
				activeParsed = false;
			
			page.find('.single-event-dates .event-load').each(function(i, e) {
				var link = {
                    title: $(this).html(),
                    active: $(this).hasClass('event-load-active'),
                    data: $(this).data()
                };
				
				if (link.active) {
					activeParsed = true;
                    eventLinks[1] = link;
				} else if (activeParsed) {
                    eventLinks[2] = link;
				} else {
                    eventLinks[i] = link;
				}
			});
			
			page.find('.single-event-dates').remove();
					
			tabs = $(App.templates['subtabs']({
				className : 'single-event-dates',
				linkClass : 'event-load',
				links : eventLinks
			}));
			page.children('.ui-header').append(tabs);

			page.trigger('create');
			
			return eventLinks;
		}
	};
	
	/* BOXSCORES RELATED FUNCTIONS */
	schedule.boxscore = {

        openedPanels : [],
        selectedTab : 0,
        boxscoresInDom : 0,
        boxscoreAPI : {},

        load : function(pageId, rawURL, isInProgress) {
            var self = this,
                url = App.normalizeURL(rawURL);

            App.initSubPage({
                pageId : pageId,
                pageType : 'boxscore',
                url : url,
                tmplData : {
                    backButtonLabel : 'Scores',
                    message : (isInProgress ? App.conf.messages.updateableBoxscore : ''),
                    gaTitle : 'Box score',
                    refreshable : true
                },
                parser : $.proxy(self.parse, self),
                callback : function(page){
                    // start auto reload
                    if (isInProgress) {
                        App.addAutoUpdatedPage(pageId, {
                            parser : $.proxy(self.parse, self)
                        });
                    }

                    self.initialize(page);

                    // removing more then several pages
                    ++ self.boxscoreInDom;
                    if (self.boxscoreInDom > App.conf.allowedBoxscores) {
                        self.clean();
                    }
                }
            });
        },

        initialize : function(page) {
            var self = this;

            self.boxscoreAPI = page.mobileBoxscore();

            page.on('pagereload', function(e, page){
                self.reloadCallback(page);
            });

            page.on('pagebeforereload', function(e, page){
                self.beforeLoadCallback(page);
            });
        },

		parse : function(html) {
			var data;
			
			if ($(html).filter('#wrapper_accordion').length) {
				data = $(html).filter('#wrapper_accordion, .message-list');
				
				//apply accordion
				data.find('#live2 .accordion_toggle').each(function(i, e) {
					var set = $(this).add(data.find('#live2 .accordion_content').eq(i));
					if (i === 0) {
						set.wrapAll('<div data-role="collapsible" data-collapsed="false" data-content-theme="d">');
					} else {
						set.wrapAll('<div data-role="collapsible" data-content-theme="d">');
					}
				});
				data.find('#live2').attr('data-role', 'collapsible-set');
				
				App.adjustImagePaths(data.find('img'));
				return helpers.outerHTML(data);
				
			} else {
				data = $('<div />').append(html);
				App.adjustImagePaths(data.find('img'));
				return data.html();
			}
		},
		
		clean : function() {
			$('.boxscore-page').not($.mobile.activePage).remove();
			this.boxscoresInDom = $('.boxscore-page').length;
		},
		
		reloadCallback : function(page) {
			this.boxscoreAPI = page.mobileBoxscore();
			
			// start with proper tab
			if (this.selectedTab !== 0) {
				this.boxscoreAPI.slide(this.selectedTab);
			}
			
			// open hidden panels back
			if (this.openedPanels.length > 0) {
				for (var panel in this.openedPanels) {
					page.find('.recent_plays').eq(this.openedPanels[panel]).prev('.toggle').get(0).click();
				}
			}
			
		},
		
		beforeLoadCallback : function(page) {
			var self = this;
			
			self.selectedTab = this.boxscoreAPI.getPos();
			
			self.openedPanels = [];
			page.find('.recent_plays').each(function(i, e){
				if ($(this).is(':visible')) {
					self.openedPanels.push(i);
				}
			});
			
		}
	}
	
})(App, jQuery)