App.services.notifications = {

	//defaults
	notificationsUpdateFlag : false,

	// init controller
	init : function() {
		var self = this,
			settings = window.localStorage.getItem('notificationsSettings'),
			storedDeviceId = window.localStorage.getItem('notificationsDeviceId');

		self.hasSettings = (settings !== null && settings.length > 0 && JSON.parse(settings).length > 0);

		// update flag
		self.notificationsUpdateFlag = window.localStorage.getItem('notificationsUpdateFlag');

		// notifications were activated previously - then plugin can be activated right away
		if (self.hasSettings || App.firstLaunch) {
			self.initPlugin();
		} else {
			// notifications had not been ever activated or app cache was cleared
			self.initPluginOnPageInit = true;
		}

		// device id received
		$.subscribe('/notifications/deviceid', function(deviceId) {
			self.deviceId = deviceId;
			window.localStorage.setItem('notificationsDeviceId', deviceId);
			// device id changed! - resyncing
			if (storedDeviceId !== null && storedDeviceId.length > 0 && storedDeviceId !== deviceId) {
				self.syncNotifications();
			}
		});

		// sync if syncing flag is true
		if (self.notificationsUpdateFlag === 'true') {
			self.syncNotifications();
		}
	},

	initPlugin : function() {
		var push;

		if (!window.device || !window.device.platform || !window.plugins || !window.PushNotification) {
			return;
		}

		push = PushNotification.init({
			"android": {
				"senderID": App.conf.plugins.genericPush.androidId
			},
			"ios": {
				"alert": "true",
				"badge": "false",
				"sound": "true"
			}
		});

		push.on('registration', $.proxy(this.onRegister, this));
		push.on('notification', $.proxy(this.onNotification, this));
	},

	initNotificationsPage : function() {
		var self = this,
			storedSettings,
			selects,
			def = new $.Deferred();

		if (! self.notificationsPageIntialized) {
			// initializing notifications plugin if no notifications settings stored

			if (self.initPluginOnPageInit) {
				self.initPlugin();
				//syncing empty list to reset, needed if user cleared app cache
				self.syncNotifications();
			}

			self.settingsPage = $('#notifications-page');
			storedSettings = JSON.parse(window.localStorage.getItem('notificationsSettings'));
			// load notifications list
			this.loadNotificationsPage().done(function(){
				var sportEnabled = false;

				selects = self.settingsPage.find('select.sport-slider');

				// update selects with locally stored preferences
				if (window.localStorage.getItem('notificationsSettings')) {
					$.each(storedSettings, function(i, e){
						if (e.value === 'true') {
							selects.filter('[name=' + e.id + ']').val('true').slider('refresh');
							sportEnabled = true;
						}
					});
				}

				// event handler for sliding triggers - serialize all data and start syncing process
				self.settingsPage.on("slidestop", function(event, ui) {
					self.notificationsListSerialized = $.map(self.settingsPage.find('select.sport-slider').serializeArray(), function(e){
						return {
							"id" : e.name,
							"value" : e.value
						};
					});
					// save preferences in local storage
					window.localStorage.setItem('notificationsSettings', JSON.stringify(self.notificationsListSerialized));

					// start syncing
					self.syncNotifications();

					if (! self.deviceId) {
						navigator.notification.alert('To enable notifications, go to settings -> notifications, select ' + App.conf.client.appName +
							' and turn ON notification center and view in lock screen', function(){}, 'Notifications error');
					}
				});

				// settings page initialized
				self.notificationsPageIntialized = true;
				def.resolve();

			}).fail(function(error) {
				// no lists added to site
				if (error === 0) {
					navigator.notification.alert('No subscriptions lists found', function(){}, 'Notifications error');
				}
				// notifications disabled for the given site
				if (error === 1) {
					navigator.notification.alert('Notifications disabled', function(){}, 'Notifications error');
				}
				def.fail();
			});

		} else {
			def.resolve();
		}
		return def;
	},

	loadNotificationsPage : function() {
		var page = this.settingsPage,
			def = new $.Deferred();

		$.publish('loading/started');
		$.getJSON(App.normalizeURL(App.conf.notificationsEndPoint), function(data){
			$.publish('loading/stopped');

			// error happened - interrupting
			if (data.error !== undefined) {
				def.reject(data.error);
				return;
			}

			page.find('.page-body').html(App.templates['notifications-page']({
				notifications : data
			}));
			page.trigger('create');

			setTimeout(function() {
				def.resolve();
			}, 100);
		});
		return def;
	},

	syncNotifications : function() {
		var self = this,
			storedSerializedList = JSON.parse(window.localStorage.getItem('notificationsSettings')),
			serializedList = '',
			ajaxDataString = $.param({
				platform  : (App.platform == 'android') ? 'gcm' : 'ios',
				id : self.deviceId,
				OSVersion : window.device.version,
				FRVersion : App.conf.ver
			}),
			syncURL = '';

		// save flag in local storage
		self.notificationsUpdateFlag = true;
		window.localStorage.setItem('notificationsUpdateFlag', self.notificationsUpdateFlag);

		// abort any previous queries
		if (self.syncQuery) {
			self.syncQuery.abort();
		}

		// clear timer if it was set
		if (self.syncTimer) {
			clearTimeout(self.syncTimer);
		}

		// prepare settings list for the GET query
		if (storedSerializedList !== null) {
			$.each(storedSerializedList, function(i, e){
				if (e.value === 'true') {
					serializedList += '&list=' + e.id;
				}
			});

			ajaxDataString += serializedList;
		}
		syncURL = App.normalizeURL(App.conf.notificationsEndPoint + '?' + ajaxDataString);

		// there is no device Id received yet - timeout syncing
		if (self.deviceId === undefined) {
			self.timeoutSyncing();
			return;
		}

		// start syncing
		$.publish('loading/started');
		self.syncQuery = $.ajax({
			'type' : 'GET',
			url : syncURL,
			success : function(data){
				self.notificationsUpdateFlag = false;
				window.localStorage.setItem('notificationsUpdateFlag',  self.notificationsUpdateFlag);
				$.publish('loading/stopped');

				// error = 2 - missing or invalid parameter, no timeout in this case
				if (data.error === 2) {
					navigator.notification.alert('There was an error while saving your preferences', function(){}, 'Notifications error');
				}

				// error = 1 - notifications disabled through a property on website
				if (data.error === 1) {
					navigator.notification.alert('Notifications are disabled', function(){}, 'Notifications error');
				}
			},
			// HTTP error - servers might be down or whatever, timeout syncing
			error : function() {
				$.publish('loading/error');
				self.timeoutSyncing();
			},
			timeout : 20000
		});
	},

	timeoutSyncing : function(){
		var self = this;

		self.syncTimer = setTimeout(function(){ self.syncNotifications(); }, 30000);
	},


	onRegister : function(data) {
		data.registrationId && $.publish('/notifications/deviceid', [data.registrationId]);
	},

	onNotification : function(data) {
		var sport,
			title = 'Notification';

		$.publish('/notifications/received');
		sport = App.getSportBySportCode(data.additionalData.open);
		if (sport) {
			title = App.conf.client.sports[sport].title;
		}

		setTimeout(function(){
			navigator.notification.alert(data.message, function() {
				if (data.additionalData.open) {
					App.swtichSportBySportCode(data.additionalData.open, 'schedule');
				}
			}, title);
		}, 0);
	}
};