(function(App, $) {

    var NavigationView = Backbone.View.extend({

        events : {
            'tap .sports-trigger' : '_sportChange',
            'tap .sub-panel-trigger' : '_triggerSubPanel',
            'tap .sub-panel-close' : '_closeSubPanel',
            'tap .icon-sport-fav' : '_sportFav',
            'tap .sport-fav' : '_sportFav',
            'tap .notifications-page-trigger' : '_openNotifications',

            'panelbeforeclose' : '_closeSubPanels',
            'panelbeforeopen' : '_resetScroll'
        },

        template : App.templates.navigation,

        openedSubPanels : [],

        initialize : function() {
            var self = this;

            _.bindAll(this, 'open', 'close', '_openSubPanel', '_toggleFavSport');

            this.iScrollHandlers = [];
            this.favSportsUpdated = false;

            this.render();
            this._initIscroll();

            $.subscribe('sports/favourite/add', self._toggleFavSport);
            $.subscribe('sports/favourite/remove', self._toggleFavSport);
        },

        render : function() {
            this.$el.html(this.template({
                favSports : App.services.favSports.getFavSports(),
                sports : App.conf.client.sports,
                showSponsorLogo : App.conf.client.showSponsorLogo || false,
                sponsorLogoURL : App.conf.client.sponsorLogoURL || '',
                currentSport : App.getCurrentSport(),
                currentModule : App.getCurrentModule(),
                website : App.conf.client.website,
                appName : App.conf.client.appName
            }));
            this._renderMoreNavbar();
        },

        refresh : function() {
            this.render();
            App.refreshLayout(this.$el);
        },

        open : function() {
            if (! this.$el.hasClass('ui-panel-open')) {
                this.$el.panel('open');
            }
        },

        close : function() {
            this.$el.panel('close');
        },

        resetHighlight : function() {
            this.$el.find('.sports-trigger').removeClass(App.conf.classes.activeBtn)
                .filter('[data-active=true]').addClass(App.conf.classes.activeBtn);
        },

        isOpened : function() {
            return this.$el.hasClass('ui-panel-open');
        },

        _sportChange : function(e) {
            var link = $(e.currentTarget),
                sport = link.data('sport'),
                module = link.data('module') || App.getCurrentModule();

            this.$el.find('.nav-list li a').removeClass(App.conf.classes.activeBtn);
            link.addClass(App.conf.classes.activeBtn);

            $.publish('/sportmenu/sportselected', [sport, module]);

            _.delay(this.close, 400);

            e.preventDefault();
        },

        _closeSubPanels : function() {
            this.$el.find('.ui-sub-panel')
                .addClass('ui-sub-panel-close ui-sub-panel-animate')
                .removeClass('ui-sub-panel-open');

            this.openedSubPanels = [];
        },

        _triggerSubPanel : function(e) {
            var link = $(e.currentTarget),
                subPanel = this.$el.find('.ui-sub-panel[data-id="' + link.data('target') + '"]');

            this._openSubPanel(subPanel);

            if (_.indexOf(this.openedSubPanels, subPanel) === -1) {
                this.openedSubPanels.push(subPanel);
            }

            e.preventDefault();
        },

        _openSubPanel : function(subPanel) {
            subPanel
                .addClass('ui-sub-panel-open ui-sub-panel-animate')
                .removeClass('ui-sub-panel-close');
        },

        _closeSubPanel : function(e) {
            var subPanel = $(e.currentTarget).closest('.ui-sub-panel');

            subPanel
                .addClass('ui-sub-panel-close ui-sub-panel-animate')
                .removeClass('ui-sub-panel-open');

            this.openedSubPanels = _.without(this.openedSubPanels, subPanel);

            e.preventDefault();
        },

        _sportFav : function(e) {
            var link = $(e.currentTarget);
            App.services.favSports.favSport(link.data('sport'));

            e.preventDefault();
        },

        _toggleFavSport : function(sport) {
            var sportFavTrigger = this.$el.find('.fav-sports-select-list').find('.icon-sport-fav[data-sport="' + sport + '"]'),
                sportFavLink = this.$el.find('.nav-list_main > .nav-sport[data-sport="' + sport + '"]');

            sportFavTrigger.toggleClass(sportFavTrigger.data('activeClass'));
            sportFavLink.toggleClass('hidden');

            if (this.isOpened()) {
                this._refreshMainScroll();
            } else {
                this.favSportsUpdated = true;
            }
        },


        _openNotifications : function(e) {
            App.services.notifications.initNotificationsPage().done(function() {
                App.changePage('notifications-page');
            });

            _.delay(this.close, 400);

            e.preventDefault();
        },

        _renderMoreNavbar : function() {
            var self = this;

            App.services.moreNavbar.getItems().then(function(data) {
                self.$el.find('.nav-list_main').append(App.templates['more-navbar']({ links : data }));
                self._refresh();
            });
        },

        _refresh : function() {
            this.$el.trigger("updatelayout");
        },

        _initIscroll : function() {
            var self = this;

            this.$el.one('panelbeforeopen', function() {
                self.$('.nav-iscroll-wrapper').each(function() {
                    self.iScrollHandlers.push(new IScroll(this, {
                        bounce : false
                    }));
                });
            });
        },

        _resetScroll : function() {
            if (! this.iScrollHandlers.length) {
                return;
            }
            _.each(this.iScrollHandlers, function(e, i) {
                e.scrollTo(0, 0);
            });

            if (this.favSportsUpdated) {
                this._refreshMainScroll();
                this.favSportsUpdated = false;
            }
        },

        // toggle first iscroll handler to update main navigation height
        _refreshMainScroll : function() {
            this.iScrollHandlers[0].refresh();
        }
    });

    _.extend(App.views, { NavigationView : NavigationView });

})(App, jQuery);