(function(App, $) {

    var pageMixin = {

        events : {
            'pagehide' : '_pageHide'
        },

        selectors : {
            pageBody : '.page-body'
        },

        /*
         * mixins functions are called after constructor functions in Cocktail
         * it means that instance properties won't be available in constructor initialize functions
         * to overcome this constructors implement _init function, that is called here after setup is completed
         * */
        initialize : function(options) {
            this.options = options;
            // initial load promise
            this.promise = $.Deferred();
            // actual paqe content
            this.page = null;
            // jQuery Mobile's ui-page wrapper
            this.uiPage = null;

            _.bindAll(this, '_init');

            if (this.options.url) {
                this.options.url = App.normalizeURL(this.options.url);

                this._load().done(this._init);
            } else {
                this._init();
            }
        },

        open : function() {
            App.changePage(this.id);
        },

        _resolve : function() {
            this.promise.resolve();
        },

        _load : function() {
            var q,
                self = this;

            $.publish('loading/started');

            switch(this.dataType) {
                case 'JSON' :
                    q = $.getJSON(this.options.url);
                    break;

                default :
                    q = $.get(this.options.url);
            }

            q.then(function(data) {
                $.publish('loading/stopped');
                self.data = data;
            }, function() {
                $.publish('loading/error');
            });

            return q;
        },

        /*
        * mixin's render called after constructor render,
        * it runs only once and wraps this.page into jquery mobile ui-page using main-page or sub-page template
        * then it calls App's createPage method to insert page into DOM and initialize with jquery mobile
        * finally Backbone's setElement method is called to update views this.el property
        * */
        render : function() {
            var template = App.templates['main-page'];

            // exit if uiPage already rendered or page not rendered
            if (! this.page || this.uiPage === this.$el) {
                return;
            }

            if (this.pageType === 'sub') {
                template = App.templates['sub-page'];
            }

            this.uiPage = $(template(_.extend({}, App.defaultTemplateData, {
                page : this.page,
                pageId : this.id,
                title : this.pageTitle,
                footerActiveButton : App.getCurrentModule(),
                url : this.options.url,
                pageClass : this.pageClass
            })));

            App.createPage(this.uiPage);
            this.setElement(this.uiPage);
        },

        _remove : function() {
            this.remove();
            App.removePage(this.id);
        },

        _refresh : function() {
            var self = this;

            if (! this.options.url) {
                return;
            }

            this._load().done(function() {
                var pageBodyContainer  = self.$(self.selectors.pageBody);

                self.render();
                pageBodyContainer.html(self.page);
                App.refreshLayout(pageBodyContainer);
                self._initUI();
            });
        }
    };

    _.extend(App.mixins, { page : pageMixin });

})(App, jQuery);